#!/usr/bin/env python

# This script will extract a raster subset using a vector mask.
# It works (well, it should) pretty much like Extract By Mask from
# the Spatial Analyst extension in ArcGIS.
# Both input datasets must be in the same projection/coordinate system.
# This script was created by Marius Jigmond (marius.jigmond@ymail.com).
# I encourage those who wish to enhance it to send me their enhancements
# to be made available to the Geo-Spatial.org community.
# Last edited: 9/2/2010

import subprocess as subp
import sys, time, os

tbegin = time.time()

if len(sys.argv) != 4: # print usage if user failed to pass arguments
  print 'Usage:\n\textract_by_mask.py <inSHP> <inRASTER> <outRASTER>'
  print 'Example:\n\textract_by_mask.py cluj.shp 30mDEM.tif cluj30mDEM.tif'
else:
  # process the shapefile to get the extents
  inSHP = sys.argv[1]
  ogrinfoOUT = subp.Popen(['ogrinfo', inSHP, inSHP.split('.')[0]], stdout = subp.PIPE).communicate()[0].split('\n')
  extentLINE = [x for x in ogrinfoOUT if 'Extent' in x][0].split(': ')[1].split(' - ')
  xmin, ymin = extentLINE[0][1:-1].split(', ')
  xmax, ymax = extentLINE[1][1:-1].split(', ')
  print 'Finished processing extents ...'

  # clip the raster to the previously determined extents
  inRAST = sys.argv[2]
  tempRAST = 'temp_rast.%s' % inRAST.split('.')[-1]
  outRAST = sys.argv[3]
  subp.call(['gdal_translate', '-projwin', xmin, ymax, xmax, ymin, inRAST, tempRAST], stdout = subp.PIPE)
  print 'Finished clipping the raster to extents ...'

  # process the actual extraction
  subp.call(['gdal_rasterize', '-burn', '9999', '-i', '-l', inSHP.split('.')[0], inSHP, tempRAST], stdout = subp.PIPE)
  subp.call(['gdal_translate', '-a_nodata', '9999', tempRAST, outRAST], stdout = subp.PIPE)
  os.remove(tempRAST)
  print 'Finished extracting by mask. Nodata = 9999'

print 'Elapsed time:', time.time() - tbegin
